/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/virtualplayqueue.h"
#include "utilities.h"      // pour randomInt
#include "yarocksettings.h" // 0.0.53
#include "debug.h"

#include "MediaItem/mediaitem.h"


/*
********************************************************************************
*                                                                              *
*    Class VirtualPlayqueue                                                    *
*                                                                              *
********************************************************************************
*/
VirtualPlayqueue::VirtualPlayqueue(QWidget *parent)
{
Q_UNUSED(parent)

    _playingIdx    = -1;
    _isPlaying     = false;
}



//! ----- add files methods -----------------------------------------------------
void VirtualPlayqueue::appendMedia(const MediaItemPtr media)
{
    Debug::debug() << "-- VirtualPlayqueue::appendMedia";
    this->clear();
    _listMedia.append( MediaItemPtr(media) );
    
    setActiveMedia(0);
    emit signal_collection_playTrack();
}

void VirtualPlayqueue::appendMedias(QList<MediaItemPtr> list, int startIdx)
{
    Debug::debug() << "-- VirtualPlayqueue::appendMedias";
    if (list.isEmpty()) return;
    
    this->clear();
    foreach(MediaItemPtr media, list)
      _listMedia.append( MediaItemPtr(media) );
      
    setActiveMedia(startIdx);
    emit signal_collection_playTrack();
}
   
    

void VirtualPlayqueue::clear()
{
    MEDIA::qResetAll( _listMedia );
    _listMedia.clear();

    _playingIdx = -1;
    _isPlaying = false;
}



MediaItemPtr VirtualPlayqueue::getNextMediaItem()
{
    //Debug::debug() << "-- VirtualPlayqueue::getNextMediaItem";
    RepeatMode  repeat_m  = (RepeatMode)SETTINGS()->_repeatMode;
    ShuffleMode shuffle_m = (ShuffleMode)SETTINGS()->_shuffleMode;
  
    //! normal mode get next track
    if(repeat_m == RepeatOff && shuffle_m == ShuffleOff) {
      if(_playingIdx < _listMedia.size()- 1) {
         setActiveMedia(++_playingIdx);
         
         return getActiveMediaItem();
      }
    }
  
    //! repeat track mode
    if(repeat_m == RepeatTrack) {
      if(_playingIdx < _listMedia.size()- 1) {
         setActiveMedia(_playingIdx);

         return getActiveMediaItem();  
      }
    }
  
    //! repeat all mode
    if(repeat_m == RepeatAll && shuffle_m == ShuffleOff) {
      if( _playingIdx == _listMedia.size()- 1) {
        setActiveMedia(0);

        return getActiveMediaItem();      
      }
      else if(_playingIdx < _listMedia.size()- 1) {
        setActiveMedia(++_playingIdx);

        return getActiveMediaItem();
      }
    }

    //! shuffle ==> random next until all is played
    if(shuffle_m == ShuffleOn) {
      QList<MediaItemPtr> list = unplayedMedia();
      if(list.isEmpty()) 
        return MediaItemPtr();
    
      int index = UTIL::randomInt(0, list.size() -1);
      MediaItemPtr media = list.at(index);
    
      setActiveMedia(_listMedia.indexOf(media));    
    
      return getActiveMediaItem();
    }

    return MediaItemPtr();
}


MediaItemPtr VirtualPlayqueue::getPrevMediaItem()
{
    //Debug::debug() << "-- VirtualPlayqueue::getPrevMediaItem";
    RepeatMode  repeat_m  = (RepeatMode)SETTINGS()->_repeatMode;
    ShuffleMode shuffle_m = (ShuffleMode)SETTINGS()->_shuffleMode;
    
    //! normal mode get prev track
    if(repeat_m == RepeatOff && shuffle_m == ShuffleOff) {
      if(_playingIdx > 0) {
        setActiveMedia(--_playingIdx);
        return getActiveMediaItem();
      }
    }

    //! repeat track mode
    if(repeat_m == RepeatTrack) {
      if(_playingIdx < _listMedia.size()- 1) {
        setActiveMedia(_playingIdx);
   
        return getActiveMediaItem();
      }
    }
  
    //! repeat all mode
    if(repeat_m == RepeatAll && shuffle_m == ShuffleOff) {
      if( _playingIdx == 0) {
        setActiveMedia(_listMedia.size()- 1);

        return getActiveMediaItem(); 
      }
      else if(_playingIdx > 0) {
        setActiveMedia(--_playingIdx);

        return getActiveMediaItem();
      }
    }
    
    //! shuffle ==> random next until all is played
    if(shuffle_m == ShuffleOn) {
      QList<MediaItemPtr> list = unplayedMedia();
      if(list.isEmpty()) 
        return MediaItemPtr();
      
      int index = UTIL::randomInt(0, list.size() -1);
      MediaItemPtr media = list.at(index);
      
      setActiveMedia(_listMedia.indexOf(media));    
      
      return getActiveMediaItem();
    }  
  
    return MediaItemPtr();
}


/* 
 VirtualPlayqueue::setActiveMedia
    - activate media in the liste at given index Idx if Idx is valid
*/
void VirtualPlayqueue::setActiveMedia(int idx)
{
    //Debug::debug() << "-- VirtualPlaylist::setActiveTrack idx=" << idx;
    
    if( (idx > -1 && idx < _listMedia.size()) ||
        (idx == 0 && _listMedia.size() == 1 ))
    {
      _listMedia.at(idx)->data[TRACK::isPlayed] = true;
      _isPlaying                = true;  
      _playingIdx               = idx;
    }
}


/* 
 VirtualPlayqueue::unplayedMedia
    - get the list of media not already played (use for shuffle mode)
*/
QList<MediaItemPtr>  VirtualPlayqueue::unplayedMedia()
{
    QList<MediaItemPtr> list;
  
    foreach (MediaItemPtr media, _listMedia) 
    {
      if(media->data.at(TRACK::isPlayed).toBool() == false)
        list << media;     
    }
  
    return list;
}
    
/* 
 VirtualPlayqueue::getActiveMediaItem
    - 
*/    
MediaItemPtr VirtualPlayqueue::getActiveMediaItem()
{
    //Debug::debug() << "-- VirtualPlaylist::getActiveMediaItem ";
    if(_playingIdx != -1)
      return _listMedia.at(_playingIdx);

    return MediaItemPtr();
}

/* 
 VirtualPlayqueue::stopPlaying
    - stop queue playing state
*/ 
void  VirtualPlayqueue::stopPlaying()
{
    _playingIdx = -1;
    _isPlaying  = false;
}


QString VirtualPlayqueue::getActiveTrack() 
{
     //Debug::debug() << "-- VirtualPlaylist::GetActiveTrack ";

     MediaItemPtr mi = getActiveMediaItem();
     if(mi)
       return mi->data.at(TRACK::url).toString();
     
     return QString();
}


QString VirtualPlayqueue::getNextTrack() 
{
     MediaItemPtr mi = getNextMediaItem();
     if(mi)
       return mi->data.at(TRACK::url).toString();
     
     return QString();
}

QString VirtualPlayqueue::getPrevTrack() 
{
     MediaItemPtr mi = getPrevMediaItem();
     if(mi)
       return mi->data.at(TRACK::url).toString();
     
     return QString();
}
