/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef MPRIS1_H
#define MPRIS1_H

#include <QDateTime>
#include <QDBusArgument>
#include <QObject>
#include <QDBusAbstractAdaptor>

#include "MediaItem/mediaitem.h"

/*******************************************************************************
    DBusStatus
*******************************************************************************/
struct DBusStatus {    // From Amarok.
  int play;            // Playing = 0, Paused = 1, Stopped = 2
  int random;          // Linearly = 0, Randomly = 1
  int repeat_track;    // Go_To_Next = 0, Repeat_Current = 1
  int repeat_playlist; // Stop_When_Finished = 0, Never_Give_Up_Playing = 1, Never_Let_You_Down = 42

  enum MprisPlayState {
    Mpris_Playing = 0,
    Mpris_Paused  = 1,
    Mpris_Stopped = 2
  };
};

Q_DECLARE_METATYPE(DBusStatus);

QDBusArgument& operator <<(QDBusArgument& arg, const DBusStatus& status);
const QDBusArgument& operator >>(const QDBusArgument& arg, DBusStatus& status);


struct Version {
  quint16 minor;
  quint16 major;
};

Q_DECLARE_METATYPE(Version);

QDBusArgument& operator <<(QDBusArgument& arg, const Version& version);
const QDBusArgument& operator >>(const QDBusArgument& arg, Version& version);





enum DBusCaps {
  NONE                 = 0,
  CAN_GO_NEXT          = 1 << 0,
  CAN_GO_PREV          = 1 << 1,
  CAN_PAUSE            = 1 << 2,
  CAN_PLAY             = 1 << 3,
  CAN_SEEK             = 1 << 4,
  CAN_PROVIDE_METADATA = 1 << 5,
  CAN_HAS_TRACKLIST    = 1 << 6,
};

/*
********************************************************************************
*                                                                              *
*    Mpris1                                                                    *
*                                                                              *
********************************************************************************
*/
class Mpris1Root;
class Mpris1Player;
class Mpris1TrackList;

class Mpris1 : public QObject
{
Q_OBJECT
  public:
    Mpris1(QObject* parent = 0);
    ~Mpris1();

    static QVariantMap GetMetadata(const MediaItemPtr);
};

/*
********************************************************************************
*                                                                              *
*    Mpris1Root                                                                *
*                                                                              *
********************************************************************************
*/
class Mpris1Root : public QObject
{
Q_OBJECT
Q_CLASSINFO("D-Bus Interface", "org.freedesktop.MediaPlayer")

  public:
    explicit Mpris1Root(QObject* parent);

  public slots:
    QString Identity();
    void Quit();
    Version MprisVersion();
};

/*
********************************************************************************
*                                                                              *
*    Mpris1Player                                                              *
*                                                                              *
********************************************************************************
*/
class Mpris1Player : public QObject
{
Q_OBJECT
Q_CLASSINFO("D-Bus Interface", "org.freedesktop.MediaPlayer")

  public:
    explicit Mpris1Player(QObject* parent);

  public slots:
    void Pause();
    void Stop();
    void Prev();
    void Play();
    void Next();
    void Repeat(bool);

    // those methods will use engine's state obtained with player->GetState() method
    DBusStatus GetStatus() const;
    int GetCaps() const;

    void VolumeSet(int volume);
    int VolumeGet() const;
    void PositionSet(int pos_msec);
    int PositionGet() const;
    QVariantMap GetMetadata() const;

    // Amarok extensions
    void VolumeUp(int vol);
    void VolumeDown(int vol);
    void Mute();

  signals:
    void CapsChange(int);
    void TrackChange(const QVariantMap&);
    void StatusChange(DBusStatus);

  private slots:
    void CurrentSongChanged();
    void EngineStateChanged();
    void ShuffleModeChanged();
    void RepeatModeChanged();

  private:
    QVariantMap       _last_metadata;
};

/*
********************************************************************************
*                                                                              *
*    Mpris1TrackList                                                           *
*                                                                              *
********************************************************************************
*/
class Mpris1TrackList : public QObject
{
Q_OBJECT
Q_CLASSINFO("D-Bus Interface", "org.freedesktop.MediaPlayer")
  public:
    explicit Mpris1TrackList(QObject* parent);

  public slots:
    int AddTrack(const QString&, bool);
    void DelTrack(int index);
    int GetCurrentTrack() const;
    int GetLength() const;
    QVariantMap GetMetadata(int) const;
    void SetLoop(bool enable);
    void SetRandom(bool enable);

    // Amarok extension
    void PlayTrack(int index);

  signals:
    void TrackListChange(int i);

  private slots:
    void PlaylistChanged();
};

#endif // _MPRIS1_H_