/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


// local
#include "mediaitem.h"
#include "Database/database.h"
#include "debug.h"

// Qt 
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>

#include <QVariant>
#include <QFileInfo>
#include <QByteArray>
#include <QBuffer>
#include <QFile>
#include <QImage>
#include <QTime>
#include <QPixmap>
#include <QCryptographicHash>

// taglib
#include <taglib/mpegfile.h>
#include <taglib/fileref.h>
#include <taglib/tstring.h>
#include <taglib/id3v2tag.h>
#include <taglib/attachedpictureframe.h>

#include <taglib/vorbisfile.h>
#include <taglib/oggfile.h>
#include <taglib/oggflacfile.h>
#include <taglib/flacfile.h>




const int SIZE_ARTIST     = 8;
const int SIZE_ALBUM      = 9;
const int SIZE_TRACK      = 21;
const int SIZE_STREAM     = 15;
const int SIZE_PLAYLIST   = 10;
const int SIZE_TRACK_CUE  = 23;


const QStringList mediaFilter    = QStringList() << "mp3" << "ogg" << "flac" << "wav" << "m4a" << "aac";
const QStringList playlistFilter = QStringList() << "m3u" << "m3u8" << "pls" << "xspf";

/*
********************************************************************************
*                                                                              *
*    Class MediaItem                                                           *
*                                                                              *
********************************************************************************
*/
MediaItem::MediaItem(T_TYPE t) : QSharedData()
{
    type = t;
    data = QVector<QVariant>( sizeForType() );  
}

MediaItem::MediaItem(const MediaItem &other) : QSharedData(other)
{
}

MediaItem::~MediaItem()
{
    foreach (MediaItemPtr mi,childItems) {
      mi.reset();
    }
    
    childItems.clear();
    data.clear();
}


MediaItemPtr MediaItem::child(int index)
{
    if (index < 0 || index >= childItems.count())
        return MediaItemPtr(0);

    return childItems.value(index);
}


int MediaItem::childCount() const
{
    return childItems.count();
}


int MediaItem::childNumber() const
{
    return 0;
}

MediaItemPtr MediaItem::addChildren(T_TYPE type)
{
    MediaItemPtr childItem = MediaItemPtr( new MediaItem(type) );
    childItems.append(childItem);
    return childItem;
}

bool MediaItem::removeChildren(int idx)
{
    if (idx < 0 || idx >= childItems.size())
        return false;

    childItems.takeAt(idx).reset();
    return true;
}

int MediaItem::sizeForType() const
{
    switch(type) {
      case TYPE_TRACK      : return SIZE_TRACK;   break;
      case TYPE_TRACK_CUE  : return SIZE_TRACK_CUE;   break;
      case TYPE_STREAM     : return SIZE_STREAM;  break;  
      case TYPE_ALBUM      : return SIZE_ALBUM;   break;    
      case TYPE_ARTIST     : return SIZE_ARTIST;  break;  
      case TYPE_PLAYLIST   : return SIZE_PLAYLIST;break;  
      case TYPE_UNKWON     : return 0;break;  
      default : return 0;break;  
    };
}

/*
********************************************************************************
*                                                                              *
*    namespace MEDIA                                                           *
*                                                                              *
********************************************************************************
*/
void MEDIA::qResetAll(const QList<MediaItemPtr> mediaList)
{
    foreach(MediaItemPtr p, mediaList)
      p.reset();
}



QString MEDIA::urlForMediaItem(const MediaItemPtr mi)
{
    QString url;
    if(mi->type == TYPE_TRACK) {
      if(!QFileInfo(mi->data.at(TRACK::url).toString()).canonicalFilePath().isEmpty())
        return QFileInfo(mi->data.at(TRACK::url).toString()).canonicalFilePath();
      else
        return QFileInfo(mi->data.at(TRACK::url).toString()).filePath();
    }
    else if(mi->type == TYPE_STREAM) 
      return mi->data.at(STREAM::url).toString();
    else
      return QString();
}

//! ---------------------- FromLocalFile ---------------------------------------
//MediaItemPtr FromLocalFile(const QString url)
MediaItemPtr MEDIA::FromLocalFile(const QString url, int* p_disc)
{
    //Debug::debug() << " ---- FromLocalFile " << url;
    //! URL check 
    if(url.isEmpty()) return MediaItemPtr(0); 

    MediaItemPtr media = MediaItemPtr(new MediaItem(TYPE_TRACK));
    media->data[TRACK::id]         = -1;    
    media->data[TRACK::url]        = QFileInfo(url).absoluteFilePath().toUtf8();
    media->data[TRACK::name]       = QFileInfo(url).baseName();
    
    //! encodedName shall be valid as long as fileName exists
    QByteArray fileName = QFile::encodeName( url );
    const char * encodedName = fileName.constData(); 

    if (!encodedName) {
      Debug::debug() << " ---- FromLocalFile -> media file access failed" << url;
      media->data[TRACK::isBroken]   = true;
      return media; 

    }
        
    //! TagLib reference 
    TagLib::FileRef fileref = TagLib::FileRef( encodedName, true);
    if (fileref.isNull()) {
      Debug::debug() << " ---- FromLocalFile -> TagLib file access Failed " << url;
      media->data[TRACK::isBroken]   = true;
      return media; 
    }
  
    //! Tag reading 
    TagLib::Tag *tag = 0;
    tag = fileref.tag();
    if ( tag ) {
        media->data[TRACK::title]      = TStringToQString(tag->title()).trimmed();
        media->data[TRACK::artist]     = TStringToQString(tag->artist()).trimmed();
        media->data[TRACK::album]      = TStringToQString(tag->album()).trimmed();
        media->data[TRACK::genre]      = TStringToQString(tag->genre()).trimmed();
        media->data[TRACK::num]        = tag->track();
        media->data[TRACK::year]       = tag->year();
    }
    
    
    //! Lenght reading
    TagLib::AudioProperties *audioProperties = fileref.audioProperties();
    if (audioProperties)             
        media->data[TRACK::duration]   = audioProperties->length(); // Returns the length of the file in seconds
    else
        media->data[TRACK::duration]   = 0;

    //! specific tags reading
    QString s_disc;
  
    if ( TagLib::MPEG::File* file = dynamic_cast<TagLib::MPEG::File *>( fileref.file() ) ) 
    {
      if ( file->ID3v2Tag() ) {
          if ( !file->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() ) 
          {
            const QString album_artist = TStringToQString( file->ID3v2Tag()->frameListMap()["TPE2"].front()->toString() ).trimmed();
            if(!album_artist.isEmpty())
             media->data[TRACK::artist] = album_artist;
          }
          
          if ( !file->ID3v2Tag()->frameListMap()["TPOS"].isEmpty() ) 
            s_disc = TStringToQString( file->ID3v2Tag()->frameListMap()["TPOS"].front()->toString() ).trimmed();
      }
    }
    else if (TagLib::Ogg::Vorbis::File* file = dynamic_cast<TagLib::Ogg::Vorbis::File*>( fileref.file() )) 
    {
      if (file->tag()) {
        const TagLib::Ogg::FieldListMap& map = file->tag()->fieldListMap();
        if (!map["ALBUMARTIST"].isEmpty()) {
          media->data[TRACK::artist] = TStringToQString( map["ALBUMARTIST"].front() ).trimmed();
        }     
        else if (!map["ALBUM ARTIST"].isEmpty()) {
          media->data[TRACK::artist] = TStringToQString( map["ALBUM ARTIST"].front() ).trimmed();
        }
        
        if (!map["DISCNUMBER"].isEmpty() )
          s_disc = TStringToQString( map["DISCNUMBER"].front() ).trimmed();
      }
    } 
    else if (TagLib::FLAC::File* file = dynamic_cast<TagLib::FLAC::File*>( fileref.file() )) 
    {
      if ( file->xiphComment() ) {
        const TagLib::Ogg::FieldListMap& map = file->xiphComment()->fieldListMap();
        if (!map["ALBUMARTIST"].isEmpty()) {
          media->data[TRACK::artist] = TStringToQString( map["ALBUMARTIST"].front() ).trimmed();
        }     
        else if (!map["ALBUM ARTIST"].isEmpty()) {
          media->data[TRACK::artist] = TStringToQString( map["ALBUM ARTIST"].front() ).trimmed();
        }
        
        if (!map["DISCNUMBER"].isEmpty() )
          s_disc = TStringToQString( map["DISCNUMBER"].front() ).trimmed();
      }
    }

    //!DEFAULT values if empty tag 
    if (media->data.at(TRACK::artist).toString().isEmpty()) 
      media->data[TRACK::artist] = QObject::tr("unknown artist");
    
    if (media->data.at(TRACK::album).toString().isEmpty()) 
      media->data[TRACK::album] = QObject::tr("unknown album");

    if (media->data.at(TRACK::genre).toString().isEmpty()) 
      media->data[TRACK::genre] = QObject::tr("unknown genre");
  
    if (media->data.at(TRACK::title).toString().isEmpty())
      media->data[TRACK::title] = media->data.at(TRACK::name);
      
    //! disk number
    if(p_disc != 0) {
      if ( !s_disc.isEmpty() ) {
        int i = s_disc.indexOf('/');
        if ( i != -1 )
          *p_disc = s_disc.left( i ).toInt();
        else
          *p_disc = s_disc.toInt();
      }
      else
        *p_disc = 0;
    }
    
    //! replaygain metadata
    MEDIA::ReplayGainTagMap map = MEDIA::readReplayGainTags( fileref );
    if ( map.contains( MEDIA::ReplayGain_Track_Gain ) )
      media->data[TRACK::trackGain] = map[MEDIA::ReplayGain_Track_Gain];
    if ( map.contains( MEDIA::ReplayGain_Track_Peak ) )
      media->data[TRACK::trackPeak] = map[MEDIA::ReplayGain_Track_Peak];
    if ( map.contains( MEDIA::ReplayGain_Album_Gain ) )
      media->data[TRACK::albumGain] = map[MEDIA::ReplayGain_Album_Gain];
    else
      media->data[TRACK::albumGain] = media->data[TRACK::trackGain];
    if ( map.contains( MEDIA::ReplayGain_Album_Peak ) )
      media->data[TRACK::albumPeak] = map[MEDIA::ReplayGain_Album_Peak];
    else
      media->data[TRACK::albumPeak] = media->data[TRACK::trackPeak];
      
    //! statistics metadata (playcount + rating)
    MEDIA::StatisticTagMap stat_map = MEDIA::readStatisticTags( fileref );
    if ( stat_map.contains( MEDIA::Rating_Tag ) )
      media->data[TRACK::rating]    = stat_map[MEDIA::Rating_Tag];
    if ( stat_map.contains( MEDIA::Playcount_Tag ) )
      media->data[TRACK::playcount] = stat_map[MEDIA::Playcount_Tag];

    //! default value    
    media->data[TRACK::isPlaying]    =  false;
    media->data[TRACK::isBroken]     =  false;
    media->data[TRACK::lastPlayed]   =  -1;
    media->data[TRACK::isPlayed]     =  false;
    media->data[TRACK::isStopAfter]  =  false;
    media->data[TRACK::playcount]    =  0;
    media->data[TRACK::rating]       =  0;
                         
    return media;
}


//! ---------------------- FromDataBase ----------------------------------------
// with track url
MediaItemPtr MEDIA::FromDataBase(const QString connectionName, const QString url)
{
  //! search track info into database
  Database db;

  //! Try database connection
  if (db.connect(connectionName)) {
    //Debug::debug() << " Media Item from db : " << url;
    QSqlField data("col",QVariant::String);
    //data.setValue(QFileInfo(url).absoluteFilePath());
    data.setValue(QFileInfo(url).canonicalFilePath());
    QString fname = db.sqlDb()->driver()->formatValue(data,false);
    
    QSqlQuery tracksQuery("SELECT id,filename,trackname, \
       number,length,artist_name,genre_name,album_name,year,last_played, \
       albumgain,albumpeakgain,trackgain,trackpeakgain,playcount,rating \
       FROM view_tracks WHERE filename="+fname+" LIMIT 1;",*db.sqlDb());
        
    if (tracksQuery.first()) {
      MediaItemPtr media = MediaItemPtr(new MediaItem(TYPE_TRACK));
      
      //Debug::debug() << " Build MediaItem FROM DATABASE ok";
      media->data[TRACK::id]         =  tracksQuery.value(0);
      media->data[TRACK::url]        =  url;
      media->data[TRACK::name]       =  tracksQuery.value(1);
      media->data[TRACK::title]      =  tracksQuery.value(2);
      media->data[TRACK::num]        =  tracksQuery.value(3);
      media->data[TRACK::duration]   =  tracksQuery.value(4);
      media->data[TRACK::artist]     =  tracksQuery.value(5);
      media->data[TRACK::genre]      =  tracksQuery.value(6);
      media->data[TRACK::album]      =  tracksQuery.value(7);
      media->data[TRACK::year]       =  tracksQuery.value(8);
      media->data[TRACK::lastPlayed] =  tracksQuery.value(9);
      media->data[TRACK::albumGain]  =  tracksQuery.value(10);
      media->data[TRACK::albumPeak]  =  tracksQuery.value(11);
      media->data[TRACK::trackGain]  =  tracksQuery.value(12);
      media->data[TRACK::trackPeak]  =  tracksQuery.value(13);
      media->data[TRACK::playcount]  =  tracksQuery.value(14);
      media->data[TRACK::rating]     =  tracksQuery.value(15);
      
      //! default value
      media->data[TRACK::isPlaying]    =  false;
      media->data[TRACK::isBroken]     =  false;
      media->data[TRACK::isPlayed]     =  false;
      media->data[TRACK::isStopAfter]  =  false;
          
      return media;
    } 
    
  } // end connect Db
  
  return MediaItemPtr(0);
}


//! ---------------------- FromDataBase ----------------------------------------
// with track index
MediaItemPtr MEDIA::FromDataBase(const QString connectionName, int trackId)
{
  //! search track info into database
  Database db;

  //! Try database connection
  if (db.connect(connectionName)) {
    
    QSqlQuery tracksQuery("SELECT id,filename,trackname, \
       number,length,artist_name,genre_name,album_name,year,last_played, \
       albumgain,albumpeakgain,trackgain,trackpeakgain,playcount,rating \
       FROM view_tracks WHERE id="+QString::number(trackId)+" LIMIT 1;",*db.sqlDb());
        
    if (tracksQuery.first()) {
      MediaItemPtr media = MediaItemPtr(new MediaItem(TYPE_TRACK));
      
      //Debug::debug() << " Build MediaItem FROM DATABASE ok";
      media->data[TRACK::id]         = tracksQuery.value(0);
      media->data[TRACK::url]        = tracksQuery.value(1);
      media->data[TRACK::name]       = tracksQuery.value(1);
      media->data[TRACK::title]      = tracksQuery.value(2);
      media->data[TRACK::num]        = tracksQuery.value(3);
      media->data[TRACK::duration]   = tracksQuery.value(4);
      media->data[TRACK::artist]     = tracksQuery.value(5);
      media->data[TRACK::genre]      = tracksQuery.value(6);
      media->data[TRACK::album]      = tracksQuery.value(7);
      media->data[TRACK::year]       = tracksQuery.value(8);
      media->data[TRACK::lastPlayed] = tracksQuery.value(9);
      media->data[TRACK::albumGain]  =  tracksQuery.value(10);
      media->data[TRACK::albumPeak]  =  tracksQuery.value(11);
      media->data[TRACK::trackGain]  =  tracksQuery.value(12);
      media->data[TRACK::trackPeak]  =  tracksQuery.value(13);
      media->data[TRACK::playcount]  =  tracksQuery.value(14);
      media->data[TRACK::rating]     =  tracksQuery.value(15);
      
      //! default value
      media->data[TRACK::isPlaying]    =  false;
      media->data[TRACK::isBroken]     =  false;
      media->data[TRACK::isPlayed]     =  false;
      media->data[TRACK::isStopAfter]  =  false;
            
      return media;
    } 
    
  } // end connect Db
  
  return MediaItemPtr(0);
}

//! -------------------------- getCoverName ------------------------------------
QString MEDIA::getCoverName(const MediaItemPtr mi)
{
  if(mi->type != TYPE_TRACK) 
    return QString();
  
  return MEDIA::getCoverName(
    mi->data.at(TRACK::artist).toString(),
    mi->data.at(TRACK::album).toString()
  );
}

QString MEDIA::getCoverName(const QString& artist,const QString& album)
{
  if( (!artist.isEmpty()) && (!album.isEmpty()) ) {
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(artist.toUtf8().constData());
    hash.addData(album.toUtf8().constData());
    
    return QString(hash.result().toHex() + ".png");
  }
  
  return QString();
}

  

//! ---------------------- getImageFromMediaUrl --------------------------------
QImage MEDIA::getImageFromMediaUrl(const QString& url, QSize size)
{
    TagLib::MPEG::File mpegFile(QFileInfo(url).filePath().toLocal8Bit());
    TagLib::ID3v2::Tag *id3tag = mpegFile.ID3v2Tag(false);
    
    if (!id3tag) {
        return QImage();
    }

    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = MEDIA::attachedPictureFrame(id3tag);
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        return QImage();
    }
    
    QByteArray pictureData = QByteArray(selectedFrame->picture().data(), selectedFrame->picture().size());
    QImage attachedImage = QImage::fromData(pictureData);
    
    if(size != attachedImage.size()) {
        attachedImage = attachedImage.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    
    return attachedImage;
}


//! ----------------------- getPixmapFromMediaUrl ------------------------------
QPixmap MEDIA::getPixmapFromMediaUrl(const QString& url, QSize size)
{
    return QPixmap::fromImage( getImageFromMediaUrl(url, size) );
}    




//! ---------------------- attachedPictureFrame --------------------------------
TagLib::ID3v2::AttachedPictureFrame *MEDIA::attachedPictureFrame(TagLib::ID3v2::Tag *id3tag, bool create)
{
    // Look for attached picture frames.
    TagLib::ID3v2::FrameList frames = id3tag->frameListMap()["APIC"];
    
    if (frames.isEmpty()) {
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    // According to the spec attached picture frames have different types.
    // So we should look for the corresponding picture depending on what
    // type of image (i.e. front cover, file info) we want.  If only 1
    // frame, just return that (scaled if necessary).
    
    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = 0;
    
    if (frames.size() != 1) {
        TagLib::ID3v2::FrameList::Iterator it = frames.begin();
        for (; it != frames.end(); ++it) {
            
            // This must be dynamic_cast<>, TagLib will return UnknownFrame in APIC for
            // encrypted frames.
            TagLib::ID3v2::AttachedPictureFrame *frame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(*it);
            
            // Both thumbnail and full size should use FrontCover, as
            // FileIcon may be too small even for thumbnail.
            if (frame && frame->type() != TagLib::ID3v2::AttachedPictureFrame::FrontCover) {
                continue;
            }
            
            selectedFrame = frame;
            break;
        }
    }
    
    // If we get here we failed to pick a picture, or there was only one,
    // so just use the first picture.
    
    if (!selectedFrame) {
        selectedFrame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(frames.front());
    }
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    return selectedFrame;
}


//! ------------------------ durationTextFromInt -------------------------------
QString MEDIA::durationTextFromInt(const int MediaItemDuration) 
{
 const int duree = (MediaItemDuration) > 0 ? MediaItemDuration : 0;
 return QTime(0,0,0,0).addSecs(duree).toString("m:ss");;
}


//! -------------------- durationTextFromMediaList -----------------------------
QString MEDIA::durationTextFromMediaList(const QList<MediaItemPtr> mediaList) 
{
    int listduration = 0;
    for (int i = 0; i < mediaList.count(); i++) {
      	MediaItemPtr mi = mediaList.at(i);
      	if(mi->type != TYPE_TRACK) {
          continue;
        }
      	else if(mi->data[TRACK::duration].toInt() > 0) {
              listduration += mi->data[TRACK::duration].toInt();
        } 
    }
    
    int hours = listduration/3600;
    int minutes = (listduration - (hours*3600))/60;
    int seconds = listduration - (hours*3600) - (minutes*60);
    QString min = minutes < 10 ? QString("0%1").arg(minutes): QString("%1").arg(minutes);
    QString sec = seconds < 10 ? QString("0%1").arg(seconds): QString("%1").arg(seconds);
    
    return QString("%1:%2:%3").arg(hours).arg(min).arg(sec);
}


/*******************************************************************************
    Media Type (local/remote)
*******************************************************************************/
bool MEDIA::isUrlLocal(const QString& url) 
{
  return (!url.contains(QRegExp("^http://")));
}

bool MEDIA::isMediaItemLocal(const MediaItemPtr mi) 
{
  return (mi->type == TYPE_TRACK);
}


/*******************************************************************************
    Media Filter
*******************************************************************************/
bool MEDIA::isAudioFile(const QString& url)
{
    return mediaFilter.contains(QFileInfo(url).suffix().toLower());
}

bool MEDIA::isPlaylistFile(const QString& url)
{
    return playlistFilter.contains(QFileInfo(url).suffix().toLower());
}

bool MEDIA::isCueFile(const QString& url)
{
    const QStringList cue_extension = QStringList() << "cue";
    return cue_extension.contains(QFileInfo(url).suffix().toLower());
}


bool MEDIA::isShoutCastUrl(const QString& url)
{
    return url.contains("pls?id");
}
 
/*******************************************************************************
    Compare/Sorting utilities
*******************************************************************************/
bool MEDIA::compareTrackItemGenre(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    const QString s1 = mi1->data.at(TRACK::genre).toString() + mi1->data.at(TRACK::album).toString();
    const QString s2 = mi2->data.at(TRACK::genre).toString() + mi2->data.at(TRACK::album).toString();
    
    return s1 < s2;
}

bool MEDIA::compareAlbumItemYear(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    return mi1->data.at(ALBUM::year).toString() > mi2->data.at(ALBUM::year).toString();
}

bool MEDIA::compareStreamName(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    return mi1->data.at(STREAM::name).toString().toLower() < mi2->data.at(STREAM::name).toString().toLower();
}

bool MEDIA::compareStreamCategorie(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    return mi1->data.at(STREAM::categorie).toString().toLower() <= mi2->data.at(STREAM::categorie).toString().toLower();
}

bool MEDIA::compareAlbumItemPlaycount(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    return mi1->data.at(ALBUM::playcount).toInt() > mi2->data.at(ALBUM::playcount).toInt();
}

bool MEDIA::compareArtistItemPlaycount(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
    return mi1->data.at(ARTIST::playcount).toInt() > mi2->data.at(ARTIST::playcount).toInt();  
}

bool MEDIA::compareAlbumItemRating(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
   return mi1->data.at(ALBUM::rating).toFloat() > mi2->data.at(ALBUM::rating).toFloat();
}

bool MEDIA::compareArtistItemRating(const MediaItemPtr mi1, const MediaItemPtr mi2)
{
   return mi1->data.at(ARTIST::rating).toFloat() > mi2->data.at(ARTIST::rating).toFloat();  
}

