/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "NowPlaying/nowplayingview.h" 
#include "EnginePlayer/engineplayer.h"
#include "MediaItem/mediaitem.h"
#include "Widget/ratingwidget.h"
#include "Database/database.h"

#include "global_actions.h"  
#include "debug.h"  


#include <QPainter>
#include <QSqlQuery>

/*
********************************************************************************
*                                                                              *
*    Class NowPlayingView                                                      *
*                                                                              *
********************************************************************************
*/
NowPlayingView::NowPlayingView(QWidget *parent) : QFrame(parent)
{
    // UI part
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor(QPalette::Text, Qt::black);
    
    this->setPalette(palette);
    this->setFrameShape(QFrame::NoFrame);
    this->setFixedHeight (130);
    this->setFocusPolicy( Qt::NoFocus );
    this->setAutoFillBackground(true);  
  
    ui_rating_widget = new RatingWidget(this);
    ui_rating_widget->setPalette(palette);
    ui_rating_widget->set_draw_frame(false);
    ui_rating_widget->setMinimumHeight(25);
    
    QSizePolicy policy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    policy.setHeightForWidth(ui_rating_widget->sizePolicy().hasHeightForWidth());
    ui_rating_widget->setSizePolicy(policy);
        
    ui_rating_widget->move(110, 95);
    ui_rating_widget->hide();

    m_context_menu        = 0;
    
    // first UI update
    update();

    //! connection
    connect(EnginePlayer::instance(), SIGNAL(playingInfoChanged()), this, SLOT(update()));
    connect(EnginePlayer::instance(), SIGNAL(streamInfoChanged()), this, SLOT(update()));
    connect(EnginePlayer::instance(), SIGNAL(engineStateChanged()), this, SLOT(update()));

    connect(ui_rating_widget, SIGNAL(RatingChanged(float)), this, SLOT(slot_rating_changed(float)));
}



/*******************************************************************************
   paintEvent
*******************************************************************************/
void NowPlayingView::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    const int width = this->width();

    //! Get data 
    MediaItemPtr media = EnginePlayer::instance()->enginePlayingMediaItem();
    QPixmap      pix   = EnginePlayer::instance()->enginePlayingPixmap();
    
    //! font
    QFont boldFont   = QFont("Segoe UI",10, QFont::Bold);
    QFont normalFont = QFont("Segoe UI", 10, QFont::Normal);
    
    painter.setPen(QColor(Qt::black));
    
    //Debug::debug() << "##### NowPlayingView::paintEvent";
  
    /*-------------------------------------------------*/
    /*  PLAYING state                                  */
    /* ------------------------------------------------*/
    if(EnginePlayer::instance()->engineState() != ENGINE::STOPPED && !EnginePlayer::instance()->enginePlayingUrl().isEmpty())
    {
      if(media->type == TYPE_TRACK)
      {
          ui_rating_widget->set_enable(media->data.at(TRACK::id).toInt() != -1);
  
          //! paint title
          painter.setFont(boldFont);
          QFontMetrics fm( boldFont);   
          const QString titleTruncated = fm.elidedText ( media->data.at(TRACK::title).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 15, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

          //! paint album    
          painter.setFont(normalFont);
          QFontMetrics fm2( normalFont);   
          const QString albumTruncated = fm2.elidedText ( media->data.at(TRACK::album).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 35, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

          //! paint artist    
          painter.setPen(QColor(0x555555));
          const QString artistTruncated = fm2.elidedText ( media->data.at(TRACK::artist).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 55, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );
    
          //! paint duration    
          if(media->data.at(TRACK::duration).toInt() > 0) {
            const QString durationText = MEDIA::durationTextFromInt(media->data.at(TRACK::duration).toInt());
            const QString dureeTruncated = fm2.elidedText ( durationText, Qt::ElideRight, width-135 );
            painter.drawText(QRect(130, 75, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,dureeTruncated );
          }
        
          //! paint track rating
          ui_rating_widget->set_rating(media->data.at(TRACK::rating).toFloat());
          ui_rating_widget->show();
      
          //! paint pixmap (coverart)
          painter.drawPixmap(10,10,110,110, pix);
      }
      else if(media->type == TYPE_STREAM)
      {
          // disable action and rating widget
          ui_rating_widget->hide();
      
          //! paint stream name
          painter.setFont(boldFont);
          QFontMetrics fm( boldFont);   
          const QString streamTruncated = fm.elidedText ( media->data.at(STREAM::name).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 15, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,streamTruncated );

          painter.setPen(QColor(0x555555));

          //! paint title    
          painter.setFont(normalFont);
          QFontMetrics fm2( normalFont);   
          const QString title_or_url = media->data.at(STREAM::title).toString().isEmpty() ? 
              media->data.at(STREAM::url).toString() : media->data.at(STREAM::title).toString();
        
          const QString titleTruncated = fm2.elidedText ( title_or_url, Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 35, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

          //! paint album    
          const QString albumTruncated = fm2.elidedText ( media->data.at(STREAM::album).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 55, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

          //! paint artist    
          const QString artistTruncated = fm2.elidedText ( media->data.at(STREAM::artist).toString(), Qt::ElideRight, width-135 );
          painter.drawText(QRect(130, 75, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

          //! paint pixmap (cover art)
          painter.drawPixmap(8,8,110,110, pix); 
      }
    }
    /*-------------------------------------------------*/
    /*  STOPPED state                                  */
    /* ------------------------------------------------*/
    else if(EnginePlayer::instance()->engineState() == ENGINE::STOPPED )  
    {
        //Debug::debug() << "##### NowPlayingView::paintEvent ###### ENGINE STOPPED";

        // disable rating widget
        ui_rating_widget->hide();
  
        // paint icon
        painter.setFont(QFont("Arial", 12, QFont::Bold));
        painter.setPen(QColor(85,85,85,100));

        painter.drawText(QRect(-10, 0, width-10, 50), Qt::AlignCenter,QString("Yarock") );
    
        int Xpos = width-64 - 20 > 0 ? (width-64)/2 -10: 10;

        painter.drawPixmap(Xpos,40,64,64, QPixmap(":/icon/64x64/yarock.png"));      
    }
    painter.end();
}


/*******************************************************************************
   contextMenuEvent
*******************************************************************************/
void NowPlayingView::contextMenuEvent(QContextMenuEvent * event)
{
    if(!m_context_menu)  
    {
      m_context_menu = new QMenu(this);
      m_context_menu->addAction(ACTIONS()->value(APP_SHOW_NOW_PLAYING));
    }
    // show the menu
    m_context_menu->popup(event->globalPos());
}


/*******************************************************************************
   slot_rating_changed
*******************************************************************************/
void NowPlayingView::slot_rating_changed(float rating)
{
    Debug::debug() << "NowPlayingView :: slot_rating_changed";
    MediaItemPtr media = EnginePlayer::instance()->enginePlayingMediaItem();
    if(media)
    {
      media->data[TRACK::rating]     =  ui_rating_widget->rating();
      int track_id = media->data.at(TRACK::id).toInt();
      
      if(track_id!=-1) {
        Database db;
        if (!db.connect("MainThread")) return;
          
        QSqlQuery q("", *db.sqlDb());
        q.prepare("UPDATE `tracks` SET `rating`=:rat WHERE `id`=:id;");
        q.bindValue(":rat", media->data.at(TRACK::rating).toFloat() );
        q.bindValue(":id", track_id );
        q.exec();
      }
    }  
}
